@@ -4,11 +4,11 @@ setClass("PsiFun", representation(n = "numeric",
                                   alpha = "numeric",
                                   c1 = "numeric"))
 
-setClass("PsiBwt", representation(M = "numeric"), 
+setClass("PsiBwt", representation(M = "numeric"),
                    contains="PsiFun")
 
 ## Define class unions for optional slots, e.g. for definition
-##  of slots which will be computed on demand, like the 
+##  of slots which will be computed on demand, like the
 ##  mahalanobis/robust distances
 setClassUnion("Uvector", c("vector", "NULL"))
 setClassUnion("Umatrix", c("matrix", "NULL"))
@@ -35,24 +35,24 @@ setClass("Cov", representation(call = "language",
                               method = "character",
                               singularity = "Ulist",
                               X = "Umatrix",
-                              "VIRTUAL")) 
+                              "VIRTUAL"))
 
 setClass("SummaryCov", representation(covobj = "Cov",
-                              evals = "vector")) 
+                              evals = "vector"))
 
-setClass("CovClassic", contains="Cov") 
+setClass("CovClassic", contains="Cov")
 
 setClass("CovRobust", representation(iter="numeric",
                                      crit="numeric",
                                      wt="Uvector",
                                      "VIRTUAL"),
-                    contains="Cov") 
+                    contains="Cov")
 
 setClass("SummaryCovRobust", representation(),
-                    contains="SummaryCov") 
+                    contains="SummaryCov")
 
 setClass("CovMest", representation(vt="vector"),
-                    contains="CovRobust") 
+                    contains="CovRobust")
 
 setClass("CovMcd", representation(alpha = "numeric",
                                   quan = "numeric",
@@ -61,15 +61,15 @@ setClass("CovMcd", representation(alpha = "numeric",
                                   raw.center = "vector",
                                   raw.mah = "Uvector",
                                   raw.wt = "Uvector",
-                                  raw.cnp2 = "numeric", 
+                                  raw.cnp2 = "numeric",
                                   cnp2 = "numeric"),
-                    contains="CovRobust") 
+                    contains="CovRobust")
 
 setClass("CovOgk", representation(raw.cov = "matrix",
                                   raw.center = "vector",
                                   raw.mah = "Uvector",
                                   raw.wt = "Uvector"),
-                    contains="CovRobust") 
+                    contains="CovRobust")
 
 setClass("CovMve", representation(alpha = "numeric",
                                   quan = "numeric",
@@ -80,12 +80,12 @@ setClass("CovMve", representation(alpha = "numeric",
                                   raw.wt = "Uvector",
                                   raw.cnp2 = "numeric",
                                   cnp2 = "numeric"),
-                    contains="CovRobust") 
+                    contains="CovRobust")
 setClass("CovSest", representation(),
-                    contains="CovRobust") 
+                    contains="CovRobust")
 
 setClass("CovSde", representation(),
-                    contains="CovRobust") 
+                    contains="CovRobust")
 
 ## Control parameters for CovMcd
 setClass("CovControlMcd", representation(alpha="numeric",
@@ -98,8 +98,8 @@ setClass("CovControlMcd", representation(alpha="numeric",
                                           trace=FALSE,
                                           tolSolve=10e-14,
                                           use.correction=TRUE),
-                           contains="CovControl") 
-                    
+                           contains="CovControl")
+
 ## Control parameters for CovMest
 setClass("CovControlMest", representation(r="numeric",
                                           arp="numeric",
@@ -112,22 +112,22 @@ setClass("CovControlMest", representation(r="numeric",
                                             trace=FALSE,
                                             tolSolve=10e-14),
                            contains="CovControl"
-) 
+)
 
 ## Control parameters for CovOgk
 ##
 ## Computes robust univariate mu and sigmma of the vector x
-##  - sigma: tau scale Yohai and Zamar (1988) - a truncated 
+##  - sigma: tau scale Yohai and Zamar (1988) - a truncated
 ##      standard deviation
 ##  - mu: weighted mean
 ##
 ## Returns a vector of length two with the calculated mu and sigma
 ##
-.mrobTau <- function(x, c1 = 4.5, c2 = 3.0, ...)       #c2=2.36075 
+.mrobTau <- function(x, c1 = 4.5, c2 = 3.0, ...)       #c2=2.36075
 {
 
     return(scaleTau2(x, mu.too=TRUE))   # use scaleTau2 from package robustbase
-    
+
 if(FALSE) {
     m0 <- median(x)                     # MED
     s0 <- median(abs(x - m0))           # MAD
@@ -135,7 +135,7 @@ if(FALSE) {
     wt <- (1 - (r/c1)^2)^2
     wt <- ifelse(r <= c1, wt, 0)        # wt = weigths w(x,c1)
     m <- sum(x*wt)/sum(wt)              # mu = weighted mean
-    
+
     r <- (x-m)/s0
     r <- r^2
     r[r > c2^2] <- c2^2                 # rho(x,c2)
@@ -146,8 +146,8 @@ if(FALSE) {
 }
 
 ##
-## Compute a robust estimate of the covariance of two random 
-##  variables x1 and x2. 
+## Compute a robust estimate of the covariance of two random
+##  variables x1 and x2.
 ## Use the estimate defined by Gnanadesikan and Kettenring (1972):
 ##     cov(X,Y)=1/4 * (sigma(X+Y)^2 - sigma(X-Y)^2)
 ##  where sigma is a robust univariate scale.
@@ -173,7 +173,7 @@ setClass("CovControlOgk", representation(niter="numeric",
                                             trace=FALSE,
                                             tolSolve=10e-14),
                            contains="CovControl"
-) 
+)
 ## Control parameters for CovMve
 setClass("CovControlMve", representation(alpha="numeric",
                                           nsamp="numeric",
@@ -183,7 +183,7 @@ setClass("CovControlMve", representation(alpha="numeric",
                                           seed=NULL,
                                           trace=FALSE,
                                           tolSolve=10e-14),
-                           contains="CovControl") 
+                           contains="CovControl")
 ## Control parameters for CovSest
 setClass("CovControlSest", representation(bdp="numeric",
                                           arp="numeric",
@@ -201,8 +201,8 @@ setClass("CovControlSest", representation(bdp="numeric",
                                             trace=FALSE,
                                             tolSolve=10e-14,
                                             method="sfast"),
-                           contains="CovControl") 
-                    
+                           contains="CovControl")
+
 ## Control parameters for CovSde
 setClass("CovControlSde", representation(nsamp="numeric",
                                           maxres="numeric",
@@ -216,8 +216,8 @@ setClass("CovControlSde", representation(nsamp="numeric",
                                           seed=NULL,
                                           trace=FALSE,
                                           tolSolve=10e-14),
-                           contains="CovControl") 
-                    
+                           contains="CovControl")
+
 
 ###################### PCA ####################################
 setClass("Pca", representation(call = "language",
@@ -232,28 +232,27 @@ setClass("Pca", representation(call = "language",
                               cutoff.od = "numeric",
                               flag = "Uvector",
                               n.obs = "numeric",
-                              "VIRTUAL")) 
+                              "VIRTUAL"))
 
-setClass("SummaryPca", representation(pcaobj = "Pca", 
-                                      importance  ="matrix")) 
+setClass("SummaryPca", representation(pcaobj = "Pca",
+                                      importance  ="matrix"))
 
-setClass("PcaClassic", contains="Pca") 
+setClass("PcaClassic", contains="Pca")
 
 setClass("PcaRobust", representation("VIRTUAL"),
-                    contains="Pca") 
+                    contains="Pca")
 
 setClass("PcaHubert", representation(alpha = "numeric",
                                   quan = "numeric"),
-                    contains="PcaRobust") 
+                    contains="PcaRobust")
 setClass("PcaLocantore", representation(),
-                    contains="PcaRobust") 
-setClass("PcaCov", representation(delta = "numeric",
-                                   quan = "numeric"),
-                                contains="PcaRobust") 
+                    contains="PcaRobust")
+setClass("PcaCov", representation(quan = "numeric"),
+                    contains="PcaRobust")
 setClass("PcaProj", representation(),
-                    contains="PcaRobust") 
+                    contains="PcaRobust")
 setClass("PcaGrid", representation(),
-                    contains="PcaRobust") 
+                    contains="PcaRobust")
 
 ###################### LDA ####################################
 setClass("Lda", representation(call = "language",
@@ -266,23 +265,23 @@ setClass("Lda", representation(call = "language",
                                method = "character",
                                X = "Umatrix",
                                grp = "factor",
-                               "VIRTUAL")) 
-                               
-setClass("SummaryLda", representation(ldaobj = "Lda")) 
+                               "VIRTUAL"))
 
-setClass("LdaClassic", contains="Lda") 
+setClass("SummaryLda", representation(ldaobj = "Lda"))
+
+setClass("LdaClassic", contains="Lda")
 
 setClass("LdaRobust", representation("VIRTUAL"),
-                    contains="Lda") 
+                    contains="Lda")
 
 setClass("PredictLda", representation(classification = "factor",
                                       posterior = "matrix",
                                       x = "matrix",
-                                      ct="Utable")) 
-                                 
-                                 
-setClass("Linda", contains="LdaRobust") 
-                                 
+                                      ct="Utable"))
+
+
+setClass("Linda", contains="LdaRobust")
+
 ###################### QDA ####################################
 setClass("Qda", representation(call     = "language",
                                prior    = "vector",
@@ -295,18 +294,18 @@ setClass("Qda", representation(call     = "language",
                                X        = "Umatrix",
                                grp      = "factor",
                                control  = "UCovControl",
-                               "VIRTUAL")) 
-                               
-setClass("SummaryQda", representation(qdaobj = "Qda")) 
+                               "VIRTUAL"))
 
-setClass("QdaClassic", contains="Qda") 
+setClass("SummaryQda", representation(qdaobj = "Qda"))
+
+setClass("QdaClassic", contains="Qda")
 
 setClass("QdaRobust", representation("VIRTUAL"),
-                    contains="Qda") 
+                    contains="Qda")
 
 setClass("PredictQda", representation(classification = "factor",
                                       posterior = "matrix",
                                       x = "matrix",
-                                      ct="Utable")) 
-                                
-setClass("QdaCov", contains="QdaRobust") 
+                                      ct="Utable"))
+
+setClass("QdaCov", contains="QdaRobust")
